
from .. import helper
from . import Exception
from django.http import JsonResponse
from django.shortcuts import redirect
from django.utils.deprecation import MiddlewareMixin
from ..management.setting import Massage, Base as BaseSetting


def handler(request, message="", status=500, save_exception=True):
    if save_exception:
        helper.cPrint('//============')
        helper.cPrint('🚨Redj Log', 'error')
        saveException(request, "http_exception", "", [], message)

    helper.cPrint(message, 'OKCYAN')
    helper.cPrint('============//')
    helper.cPrint('', 'ENDC')

    if BaseSetting.response_type == 'url':
        return redirect(BaseSetting.response_url+'?status='+str(status)+'&message='+message)
    else:
        response = {}
        try:
            if BaseSetting.message_key:
                response[BaseSetting.message_key] = message
            if BaseSetting.status_key:
                response[BaseSetting.status_key] = status
        except:
            pass
        return JsonResponse(response, status=status)


class Base(MiddlewareMixin):
    def process_exception(self, request, exception):
        helper.cPrint('//============')
        if request == None:
            helper.cPrint('⚠️ Redj Log', 'OKBLUE')
        else:
            helper.cPrint('🚨Redj Log', 'error')
        helper.cPrint('', 'ENDC')

        type = 'Other'
        tracebacks = []
        exception_str = ''

        try:
            exception_str = str(exception)
            if exception_str:
                helper.cPrint(exception_str, 'OKCYAN')
                helper.cPrint('', 'ENDC')
        except:
            pass

        try:
            tb = exception.__traceback__
            while tb is not None:
                if "env" not in tb.tb_frame.f_code.co_filename:
                    tracebacks.append({
                        "name": tb.tb_frame.f_code.co_name,
                        "filename": tb.tb_frame.f_code.co_filename,
                        "lineno": tb.tb_lineno
                    })
                tb = tb.tb_next
            if BaseSetting.debug:
                print(tracebacks)
        except:
            pass

        status = 500
        message = Massage.other_massage
        if(isinstance(exception, Exception.DateException)):
            type = 'DateException'
            message = Massage.date_massage
        if(isinstance(exception, Exception.AuthException)):
            status = 401
            type = 'AuthException'
            message = Massage.auth_massage
        if(isinstance(exception, Exception.OtherException)):
            type = 'OtherException'
            message = str(exception)
        if(isinstance(exception, Exception.UploadException)):
            type = 'UploadException'
            message = Massage.upload_massage
        if(isinstance(exception, Exception.BlockIpException)):
            status = 403
            type = 'BlockIpException'
            message = Massage.block_massage
        if(isinstance(exception, Exception.ValidatorException)):
            status = 412
            type = 'ValidatorException'
            message = Massage.validator_massage
        if(isinstance(exception, Exception.PermissionException)):
            status = 403
            message = "Access denied"
            type = Massage.permission_massage
        if(isinstance(exception, Exception.DuplicateException)):
            status = 412
            type = 'DuplicateException'
            message = Massage.duplicate_massage
        if(isinstance(exception, Exception.NotFoundException)):
            status = 404
            message = "Not Found"
            type = Massage.not_found_massage

        saveException(request, type, exception_str, tracebacks, message)
        return handler(request, message, status, False)


def saveException(request, type, exception, tracebacks, message):
    data = {
        'path': '',
        'method': '',
        'user_id': 0,
        'type': type,
        'file_path': '',
        'user_agent': '',
        'line_number': 0,
        'base_url': None,
        'full_path': None,
        'message': message,
        'remote_address': '',
        'exception': exception
    }

    try:
        if len(tracebacks) > 0:
            data['file_path'] = tracebacks[len(tracebacks)-1]['filename']
            data['line_number'] = tracebacks[len(tracebacks)-1]['lineno']
    except:
        pass

    try:
        data['method'] = request.method
        data['full_path'] = request.get_full_path()
        data['base_url'] = "{0}://{1}{2}".format(
            request.scheme, request.get_host(), request.path)

        meta = request.META.copy()
        data['user_agent'] = meta.pop('HTTP_USER_AGENT', None)
        data['remote_address'] = meta.pop('REMOTE_ADDR', None)
    except:
        pass

    try:
        if hasattr(request, 'user'):
            if request.user.is_authenticated:
                data['user_id'] = request.user.id
        else:
            data['user_id'] = 0
    except:
        pass

    try:
        data['client_key'] = request.redjlog['request_id']
    except:
        pass

    try:
        helper.sendToServer(data, 'exception')
    except:
        helper.cPrint('Bad Error')
        pass


def catch(ex):
    try:
        handl = Base()
        handl.process_exception(None, ex)
    except:
        pass



import string
import random
from .management.server import reqApi


def cPrint(massage, color="warning"):
    print(printColor(color) + massage + printColor(color))


def printColor(color):
    colors = {
        "warning": '\033[93m',
        "error": '\033[91m',
        "HEADER": '\033[95m',
        "OKBLUE": '\033[94m',
        "OKCYAN": '\033[96m',
        "OKGREEN": '\033[92m',
        "WARNING": '\033[93m',
        "FAIL": '\033[91m',
        "ENDC": '\033[0m',
        "BOLD": '\033[1m',
        "UNDERLINE": '\033[4m'
    }

    return colors[color]


def sendToServer(data, type, config=None):
    return reqApi(data, type, config)


def randomString(size=40):
    text = string.digits
    text += string.punctuation
    text += string.ascii_letters
    text += string.ascii_lowercase
    text += string.ascii_uppercase

    return ''.join(random.choice(text) for i in range(size))



import json
import requests
import threading
from .setting import Base

api_list = {
    'request': '/request/store/',
    'exception': '/exception/store/',
    'request_update': '/request/edit/',
}


def reqApi(data, type, config=None):
    try:
        if not Base.is_valid():
            return
        if not api_list[type]:
            return

        data['token'] = Base.api_key
        data['project'] = Base.project_key
        url = Base.server_url+api_list[type]
        data = json.dumps(data)
        headers = {
            'Content-Type': 'application/json'
        }
        if config and config['pk']:
            url += str(config['pk'])+'/'
        threading.Thread(
            target=reqApiBase,
            args=(url, data, headers)
        ).start().join()
        return
    except:
        pass


def reqApiBase(url, data, headers):
    try:
        requests.post(url, data=data, headers=headers,timeout=0.0000000001)
    except :
        pass



from django.http import response


class Base:
    debug = False
    api_key = None
    server_url = None
    response_url = "/"
    project_key = None
    status_key = "status"
    response_type = "json"
    message_key = "message"

    def is_valid():
        if Base.api_key == None:
            return False
        if Base.server_url == None:
            return False
        if Base.project_key == None:
            return False

        if len(Base.api_key) < 20:
            return False
        if "http" not in Base.server_url:
            return False

        return True


class Massage:
    not_found_massage = "Not Found"
    auth_massage = "Illegal access"
    block_massage = "your are block"
    permission_massage = "Access denied"
    upload_massage = "Error in upload file"
    date_massage = "Error in recorded dates"
    duplicate_massage = "Item already exists"
    validator_massage = "Fields are not complete"
    other_massage = "The server cannot respond at this time"


def init(api_key, project_key, server_url, debug=False, response_url=None, response_type='json', status_key='status', message_key='message'):
    Base.debug = debug
    Base.api_key = api_key
    Base.server_url = server_url
    Base.project_key = project_key
    Base.response_url = response_url
    Base.response_type = response_type
    Base.status_key = status_key.replace(" ", "_")
    Base.message_key = message_key.replace(" ", "_")


def exceptionMessage(
        auth_massage=None,
        date_massage=None,
        block_massage=None,
        other_massage=None,
        upload_massage=None,
        duplicate_massage=None,
        validator_massage=None,
        not_found_massage=None,
        permission_massage=None):
    if auth_massage:
        Massage.auth_massage = auth_massage
    if date_massage:
        Massage.date_massage = date_massage
    if block_massage:
        Massage.block_massage = block_massage
    if other_massage:
        Massage.other_massage = other_massage
    if upload_massage:
        Massage.upload_massage = upload_massage
    if duplicate_massage:
        Massage.duplicate_massage = duplicate_massage
    if validator_massage:
        Massage.validator_massage = validator_massage
    if not_found_massage:
        Massage.not_found_massage = not_found_massage
    if permission_massage:
        Massage.permission_massage = permission_massage



try:
    from .Exception import AuthException
    from knox.auth import TokenAuthentication

    class Authentication(TokenAuthentication):
        def authenticate_credentials(self, token):
            try:
                return super().authenticate_credentials(token)
            except:
                raise AuthException()
except:
    pass



from .ExceptionHandler import handler
from ..management.setting import Massage


def handler400(request, exception):
    message = Massage.auth_massage
    return handler(request, message, 400)


def handler403(request, exception):
    message = Massage.permission_massage
    return handler(request, message, 403)


def handler404(request, exception):
    message = Massage.not_found_massage
    return handler(request, message, 404)


def handler500(request):
    message = Massage.other_massage
    return handler(request, message)



import time
from .. import helper
from django.utils.deprecation import MiddlewareMixin


class Base(MiddlewareMixin):
    def __call__(self, request):
        start_time = time.time()
        request_id = self.saveRequest(request)
        request.redjlog={"request_id":request_id}

        response = self.get_response(request)

        exec_time = int((time.time() - start_time)*1000)
        self.updateRequest(response, request, exec_time, request_id)
        return response

    def saveRequest(self, request):
        request_id = helper.randomString()
        data = {
            'ip': None,
            'user_id': 0,
            'method': None,
            'is_ajax': None,
            'base_url': None,
            'full_path': None,
            'user_agent': None,
            'body_request': None,
            'remote_address': None,
            'client_key': request_id
        }
        try:
            data['method'] = request.method
            data['is_ajax'] = request.is_ajax()
            data['full_path'] = request.get_full_path()
            data['base_url'] = "{0}://{1}{2}".format(request.scheme, request.get_host(), request.path)

            meta = request.META.copy()
            data['user_agent'] = meta.pop('HTTP_USER_AGENT', None)
            data['remote_address'] = meta.pop('REMOTE_ADDR', None)
        except:
            pass

        try:
            data['ip'] = self.getClientIP(request)
        except:
            pass

        try:
            data['body_request'] = str(request.body)
        except:
            pass

        try:
            if hasattr(request, 'user'):
                if request.user.is_authenticated:
                    data['user_id'] = request.user.id
            else:
                data['user_id'] = 0
        except:
            pass

        try:
            helper.sendToServer(data, 'request')
        except:
            pass

        return request_id

    def updateRequest(self, response, request, exec_time, request_id):
        data = {
            'user_id': 0,
            'response_code': 0,
            'exec_time': exec_time,
            'client_key': request_id,
        }
        try:
            data['response_code'] = response.status_code
        except:
            pass

        try:
            if hasattr(request, 'user'):
                if request.user.is_authenticated:
                    data['user_id'] = request.user.id
            else:
                data['user_id'] = 0
        except:
            pass

        try:
            # send request
            helper.sendToServer(data, 'request_update')
            pass
        except:
            pass

    def getClientIP(self, request):
        client_ip = ''
        try:
            x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
            if x_forwarded_for:
                client_ip = x_forwarded_for.split(',')[0]
            else:
                client_ip = request.META.get('REMOTE_ADDR')
        except:
            pass
        return client_ip



class PayException(Exception):
    pass


class DateException(Exception):
    pass


class AuthException(Exception):
    pass


class OtherException(Exception):
    pass


class NotFoundException(Exception):
    pass


class UploadException(Exception):
    pass


class BlockIpException(Exception):
    pass


class ValidatorException(Exception):
    pass


class DuplicateException(Exception):
    pass


class PermissionException(Exception):
    pass


from .exception.Exception import *
from .exception.ExceptionHandler import catch
from .management.setting import init, exceptionMessage

__all__ = [
    'init',
    'catch',
    "PayException",
    "DateException",
    "AuthException",
    "OtherException",
    "UploadException",
    "BlockIpException",
    'exceptionMessage',
    "NotFoundException",
    "ValidatorException",
    "DuplicateException",
    "PermissionException"
]
